from django import forms
from django.contrib.auth.forms import UserCreationForm
from django.middleware.csrf import get_token
from django.utils.html import format_html
from django.utils.safestring import mark_safe

from django_extension.forms import HyperModelForm


class HyperFormMixin:

    def __init__(self, submit='def', cssclass='', *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.request = kwargs.pop('request', None)
        self.submit = submit
        self.css_class = cssclass

    def as_ht(self):

        csrf_t = 'None'
        submit = self.submit
        cssclass = format_html(u'class="{}"', self.css_class) if len(self.css_class) > 0 else ''

        if self.request != None:
            csrf_t = '<input type="hidden" name="csrfmiddlewaretoken" value="' + get_token(self.request) + '">'
        if self.submit == 'def':
            submit = 'in_' + self.__class__.__name__ + '_notdefined'

        html = '<form method="post" ' + cssclass + '>' + csrf_t + self.as_p() + '<input type="submit" value=' + submit + '></form>'

        return mark_safe(html)

    def __unicode__(self):
        return self.as_h()


"""
class SignUpForm(UserCreationForm, DivMixinForm):
    email = forms.EmailField(required=True, label='Email')

    class Meta:
        model = Profile
        fields = ("username", "email", "password1", "password2")

    def __init__(self, *args, **kwargs):
        super(SignUpForm, self).__init__(*args, **kwargs)
        self.submit = u'Присоединиться'

    def save(self, commit=True):
        user = super(SignUpForm, self).save(commit=False)
        user.email = self.cleaned_data["email"]
        if commit:
            user.save()
        return user
"""



# -*- coding: utf-8 -*-
import datetime
# from distutils.version import StrictVersion


from django import get_version, forms
from django.forms import Widget
from django import utils

from django.forms.utils import flatatt

import json  # # import simplejson as json
import copy


class JSONViewWidget(forms.Widget):
    # class Media:
    #     # css = {'all': ("iri.css",)}
    #     js = ("js/auto_url_input.js",)

    label_def_style = 'style="display: inline-block;float:none;"'
    input_def_style = 'style="width:50vw;"'

    def __init__(self, attrs=None, newline='<br/>\n', sep='__', debug=False):
        self.newline = newline
        self.separator = sep
        self.debug = debug
        Widget.__init__(self, attrs)

    def _label_render(self, attrs, key):
        return f"""<label {self.label_def_style} for="{attrs['id']}">{key}:</label>"""

    def _delete_btn(self, key):
        btn = f"""<div class='del_key' onclick='del_key(event, "{key}")'></div>"""
        return btn

    def _append_value(self):
        btn = f"""<div class='append_value' onclick='append_value(event)'></div>"""
        return btn

    def _as_text_field(self, name, key, value, is_sub=False):
        attrs = self.build_attrs(self.attrs, {
            "type": 'text',
            "name": "%s%s%s" % (name, self.separator, key),
        })
        attrs['value'] = utils.encoding.force_text(value)
        attrs['id'] = attrs.get('name', None)
        input_def_style = self.input_def_style[:-1] + 'margin-left:-3px;"'
        return f"""
                <label {self.label_def_style} for="{attrs['id']}">{key}:</label>
                <input{flatatt(attrs)}  {input_def_style} />
                {self._delete_btn(attrs['id'])}"""

    def _as_textarea_field(self, name, key, value, is_sub=False):
        attrs = self.build_attrs(self.attrs, {
            "name": "%s%s%s" % (name, self.separator, key),
        })
        attrs['id'] = attrs.get('name', None)
        return f"""
            <label for="{attrs['id']}">{key}:</label>
            <textarea{flatatt(attrs)} {self.input_def_style} rows=10>{utils.encoding.force_text(value)}
            </textarea>
        """

    def _as_number_field(self, name, key, value, is_sub=False):
        attrs = self.build_attrs(self.attrs, {
            "type": 'number',
            "name": "%s%s%s" % (name, self.separator, key),
        })
        attrs['value'] = utils.encoding.force_text(value)
        attrs['id'] = attrs.get('name', None)
        return f"""<label  {self.label_def_style} for="{attrs['id']}">{key}:</label><input{flatatt(attrs)} />"""

    def _as_float_field(self, name, key, value, is_sub=False):
        attrs = self.build_attrs(self.attrs, {
            "type": 'number',
            "step": '0.01',
            "name": "%s%s%s" % (name, self.separator, key),
        })
        attrs['value'] = utils.encoding.force_text(value)
        attrs['id'] = attrs.get('name', None)
        return f"""
            <label for="{attrs['id']}">{key}:</label>
            <input{flatatt(attrs)} />
        """

    def _as_date_field(self, name, key, value, is_sub=False):
        attrs = self.build_attrs(self.attrs, {
            "type": 'date',
            "name": "%s%s%s" % (name, self.separator, key),
        })
        attrs['value'] = utils.encoding.force_text(value)
        attrs['id'] = attrs.get('name', None)
        return f"""
            <label for="{attrs['id']}">{key}:</label>
            <input{flatatt(attrs)} />
        """

    def _as_datetime_field(self, name, key, value, is_sub=False):
        attrs = self.build_attrs(self.attrs, {
            "type": 'datetime-local',
            "name": "%s%s%s" % (name, self.separator, key),
        })
        attrs['value'] = utils.encoding.force_text(value)
        attrs['id'] = attrs.get('name', None)
        return f"""
            <label for="{attrs['id']}">{key}:</label>
            <input{flatatt(attrs)} />
        """

    def _as_checkbox_field(self, name, key, value, is_sub=False):
        classes = self.attrs['class']
        classes = [c for c in classes.split() if c != 'form-control']
        classes.append("switch")
        attrs = self.build_attrs(self.attrs, {
            "type": 'checkbox',
            "class": " ".join(classes),
            "onclick": "self.previousSibling.value=!JSON.parse(self.previousSibling.value)",
        })
        hidden_name = "%s%s%s" % (name, self.separator, key)
        attrs['value'] = utils.encoding.force_text(value)
        attrs['id'] = attrs.get('name', None)
        checked = '' if not bool(value) else 'checked'
        return f"""
            <label {self.label_def_style} for="{attrs['id']}">{key}:</label>
            <input type="hidden" name="{hidden_name}" value="{attrs['value'].lower()}"><input{flatatt(attrs)} {checked}/>
        """

    def _as_select_field(self, name, key, value, is_sub=False):
        attrs = self.build_attrs(self.attrs, {
            "size": "none",
            "name": "%s%s%s" % (name, self.separator, key),
        })
        # attrs['value'] = utils.encoding.force_text(value)
        attrs['id'] = attrs.get('name', None)
        # attrs['disabled'] = True
        attrs.pop('name', None)  # todo (make readonly until todo valid multiple params come in)

        label = f"""<label for="{attrs['id']}">{key}:</label>"""

        options = ''
        for option in value:
            options += f'<option selected ondblclick="append_value(event, false)" value="{option}">{option}</option>'

        input_def_style = self.input_def_style[:-1] + 'margin-top: 3px;"'
        return f"""
                {label}<select{flatatt(attrs)} class='jsn_array' multiple 
                {input_def_style}>{options}</select>
                {self._delete_btn(attrs['id'])}
                {self._append_value()}"""

    def _to_build(self, name, json_obj):
        """
        Генерирует дерево инпутов
        :param name:
        :param json_obj: объект json
        :return:
        """
        inputs = []
        if isinstance(json_obj, list):
            if all(isinstance(item, (str, int, float, bool)) for item in json_obj):
                name, _, key = name.rpartition(self.separator)
                inputs.append(self._as_select_field(name, key, json_obj))
            else:
                title = name.rpartition(self.separator)[2]
                _l = []  # ['%s:%s' % (title, self.newline)]
                for key, value in enumerate(json_obj):
                    _l.append(self._to_build("%s%s%s" % (name, self.separator, key), value))
                inputs.extend([_l])
        elif isinstance(json_obj, dict):
            title = name.rpartition(self.separator)[2]
            _l = []  # ['%s:%s' % (title, self.newline)]
            # inputs.extend('<div>Заголовок</div>')
            for key, value in json_obj.items():
                if value != {}:
                    _l.append(self._to_build("%s%s%s" % (name, self.separator, key), value))
            inputs.extend([_l])
        elif isinstance(json_obj, str):
            name, _, key = name.rpartition(self.separator)
            if len(json_obj) > 50:
                inputs.append(self._as_textarea_field(name, key, json_obj))
            else:
                inputs.append(self._as_text_field(name, key, json_obj))
        elif isinstance(json_obj, bool):
            name, _, key = name.rpartition(self.separator)
            inputs.append(self._as_checkbox_field(name, key, json_obj))
        elif isinstance(json_obj, int):
            name, _, key = name.rpartition(self.separator)
            inputs.append(self._as_number_field(name, key, json_obj))
        elif isinstance(json_obj, float):
            name, _, key = name.rpartition(self.separator)
            inputs.append(self._as_float_field(name, key, json_obj))
        elif isinstance(json_obj, datetime.date):
            name, _, key = name.rpartition(self.separator)
            inputs.append(self._as_date_field(name, key, json_obj))
        elif isinstance(json_obj, datetime.datetime):
            name, _, key = name.rpartition(self.separator)
            inputs.append(self._as_datetime_field(name, key, json_obj))
        elif json_obj is None:
            name, _, key = name.rpartition(self.separator)
            inputs.append(self._as_text_field(name, key, ''))
        return inputs if len(inputs) > 1 else inputs[0]

    def _prepare_as_div(self, input_list, level):
        """
        Форматирование дерева инпутов
        :param input_list:
        :param level:
        :return:
        """
        form_group_class = ''
        input_list = [input_list] if type(input_list) is str else input_list
        if input_list:
            result = ''
            for item in input_list:
                class_ex = ''
                if level == 0 or level == 1:
                    class_ex = 'nested'

                if isinstance(item, list) and len(input_list) == 1:
                    result += '%s' % self._prepare_as_div(item, level)
                elif isinstance(item, list):
                    if level == 0:
                        result += f'<div class="{form_group_class} {class_ex}">'
                        result += '%s' % self._prepare_as_div(item, level + 1)
                        result += '</div>'
                    else:
                        result += f'<div class="{form_group_class} {class_ex}">'
                        result += '%s' % self._prepare_as_div(item, level + 1)
                        result += '</div>'
                else:
                    if level == 1:
                        level_title = ''  # '<p>hhhhhhh</p>'
                        result += '%s<div class="%s col-md-12">%s</div>' % (level_title, form_group_class, item)
                    else:
                        result += '<div class="%s">%s</div>' % (form_group_class, item)
            return result
        return ''

    def _to_pack_up(self, root_node, raw_data):

        copy_raw_data = copy.deepcopy(raw_data)
        result = []

        def _to_parse_key(k, v):
            if k.find(self.separator) != -1:
                apx, _, nk = k.rpartition(self.separator)
                if apx.startswith('__'):
                    apx = apx[2:]

                try:
                    # parse list
                    int(nk)
                    l = []
                    obj = {}
                    index = None
                    if apx != root_node:
                        for key, val in list(copy_raw_data.items()):
                            head, _, t = key.rpartition(self.separator)
                            _, _, index = head.rpartition(self.separator)
                            if key is k:
                                del copy_raw_data[key]
                            elif key.startswith(apx):
                                try:
                                    int(t)
                                    l.append(val)
                                except ValueError:
                                    if index in obj:
                                        obj[index].update({t: val})
                                    else:
                                        obj[index] = {t: val}
                                del copy_raw_data[key]
                        if obj:
                            for i in obj:
                                l.append(obj[i])
                    l.append(v)
                    return _to_parse_key(apx, l)
                except ValueError:
                    # parse dict
                    d = {}
                    if apx != root_node:
                        for key, val in list(copy_raw_data.items()):
                            _, _, t = key.rpartition(self.separator)
                            try:
                                int(t)
                                continue
                            except ValueError:
                                pass
                            if key is k:
                                del copy_raw_data[key]
                            elif key.startswith(apx):
                                d.update({t: val})
                                del copy_raw_data[key]
                    v = {nk: v}
                    if d:
                        v.update(d)
                    return _to_parse_key(apx, v)
            else:
                return v

        for k, v in list(raw_data.items()):
            if k in copy_raw_data:
                # to transform value from list to string
                v = v[0] if isinstance(v, list) and len(v) is 1 else v
                if k.find(self.separator) != -1:
                    d = _to_parse_key(k, v)
                    # set type result
                    if not len(result):
                        result = type(d)()
                    try:
                        result.extend(d)
                    except:
                        result.update(d)
        return result

    def value_from_datadict(self, data, files, name):
        """
        распаковка для валидации
        :param data:
        :param files:
        :param name:
        :return:
        """
        data_copy = copy.deepcopy(data)
        result = self._to_pack_up(name, data_copy)
        return json.dumps(result)

    def render(self, name, value, attrs=None, renderer=None):
        try:
            value = json.loads(value)
        except (TypeError, KeyError):
            pass
        inputs = self._to_build(name, [] if value == [] else (value or {}))
        result = self._prepare_as_div(inputs, 0)

        append_btn = f'<div class="append_btn" onclick="add_json_field(event)"></div>'
        result = f'<div class="json_container" style="float:left" class="row">{result}{append_btn}</div>'

        # add_json_field        '

        if self.debug:
            # render json as well
            source_data = u'<hr/>Source data: <br/>%s<hr/>' % str(value)
            result = '%s%s' % (result, source_data)
        return utils.safestring.mark_safe(result)

    # template_name =
    #
    # def render(self, name, value, attrs=None, renderer=None):
    #     """Render the widget as an HTML string."""
    #     context = self.get_context(name, value, attrs)
    #     return self._render(self.template_name, context, renderer)

    class Media:
        css = {'all': ("style/json_field.css",)}
        js = ("js/json_field.js",)



from django import forms
from django.forms import Widget, MultiWidget, SelectDateWidget
from django.forms.widgets import ChoiceWidget, ClearableFileInput, \
    FileInput, FILE_INPUT_CONTRADICTION, URLInput

f = forms.widgets.CheckboxInput
f = forms.widgets.SelectMultiple
f = forms.widgets.Select


class AutoUrlInput(URLInput):
    class Media:
        # css = {'all': ("iri.css",)}
        js = ("js/auto_url_input.js",)

    template_name = 'forms/widgets/auto_url.html'
    oninput = 'urlFieldPreFill'

    def __init__(self, attrs=None, oninput=None):
        super().__init__(attrs)
        self.attrs['oninput'] = oninput or self.oninput + '(event)'


"""
class ToggleWidget(Widget):
    template_name = 'django/forms/widgets/checkbox.html'
    option_template_name = 'django/forms/widgets/checkbox.html'

    class Media:
        css = {'all': ("",)}
        js = ("",)

    def __init__(self, attrs=None, *args, **kwargs):
        attrs = attrs or {}

        default_options = {
            'toggle': 'toggle',
            'offstyle': 'danger'
        }
        options = kwargs.get('options', {})
        default_options.update(options)
        for key, val in default_options.items():
            attrs['data-' + key] = val

        super().__init__(attrs)
"""


class ImageWidget(FileInput):
    template_name = 'forms/widgets/image_picker.html'

    def format_value(self, value):
        """
        Return the file object if it has a defined url attribute.
        """
        if self.is_initial(value):
            return value

    def is_initial(self, value):
        """
        Return whether value is considered to be initial value.
        """
        return bool(value and getattr(value, 'url', False))

    def get_context(self, name, value, attrs):
        context = super().get_context(name, value, attrs)
        context['widget'].update({
            'is_initial': self.is_initial(value),
            'jar_class': '',
            'img_class': '',
            'clr_class': ''
        })
        return context

    # почти дефолтное поведение:

    def value_from_datadict(self, data, files, name):
        upload = super().value_from_datadict(data, files, name)
        return upload

    def use_required_attribute(self, initial):
        return super().use_required_attribute(initial) and not initial

    def value_omitted_from_data(self, data, files, name):
        return (
            super().value_omitted_from_data(data, files, name)
        )


class DynamicMultiSelect(ChoiceWidget):  # ChoiceWidget

    duty_symbols = {
        '+': 'plus',
        '#': 'sharp'
    }

    template_name = 'forms/widgets/dynamic_multi_select.html'

    # ChoiceWidget:
    # allow_multiple_selected = True
    # optgroups = ChoiceWidget.optgroups
    # create_option = ChoiceWidget.create_option
    #
    # format_value = ChoiceWidget.format_value  # форматирует вывод. Исп в Widget
    #
    # subwidgets = ChoiceWidget.subwidgets
    # options = ChoiceWidget.options
    #
    # __deepcopy__ = ChoiceWidget.__deepcopy__
    # id_for_label = ChoiceWidget.id_for_label

    def __init__(self, url='', placeholder=None, attrs=None, **kwargs):  # choices=(), # # ChoiceWidget
        if placeholder:
            attrs = attrs or {}
            attrs['placeholder'] = placeholder
        super().__init__(attrs, **kwargs)
        self.url = url
        # self.choices = choices  # ChoiceWidget

    def get_context(self, name, value, attrs):

        # from django.forms.models import ModelChoiceIterator
        # ModelChoiceIterator

        def_attrs = ['id', 'required', 'multiple']  # multiple todo in template
        default_attrs = {key: attrs.pop(key) for key in def_attrs if key in attrs}

        context = super().get_context(name, value, attrs)
        # ChoiceWidget:
        # context['widget']['optgroups'] = self.optgroups(name, context['widget']['value'], attrs)

        context['widget']['url'] = self.url
        context['default_attrs'] = default_attrs
        return context

    def value_from_datadict(self, data, files, name):
        return data.getlist(name)

    def create_option(self, name, value, label, *args, **kwargs):
        options = super().create_option(name, value, label, *args, **kwargs)
        options['attrs'].update({
            'data-id': value,
            'data-hint': label,
            'selected': True,

            'in_swarm_id': name + '_tag_' + self.valid_id_format(label),
        })
        return options

    # not inheritance:
    def valid_id_format(self, name: str):
        for char in self.duty_symbols:
            name = name.replace(char, self.duty_symbols[char])
        return name



from django.forms import DateInput, TimeInput


class DateWidget(DateInput):
    template_name = 'forms/widgets/date.html'

    def __init__(self, placeholder='', attrs=None, format=None):
        # attrs['type'] = 'date'
        super().__init__(attrs, format)
        self.placeholder = placeholder

    def get_context(self, name, value, attrs):
        context = super().get_context(name, value, attrs)
        context['widget']['type'] = 'date'  # self.input_type
        context['placeholder'] = self.placeholder or 'Введите дату'
        return context


class TimeWidget(TimeInput):
    template_name = 'forms/widgets/time.html'



from django import forms
from django.core.validators import EmailValidator
from django.core.exceptions import NON_FIELD_ERRORS

# для кастомного Field:
from django.forms import BoundField, Field, ImageField, ModelForm, FileField
from django.utils.translation import ugettext_lazy as _
from django.utils.html import format_html, conditional_escape
from django.forms.utils import flatatt
from django.utils.safestring import mark_safe

from django.middleware.csrf import get_token


class HyperModelForm(ModelForm):
    action = ''
    submit = None

    def __init__(self, submit='def', cssclass='', *args, **kwargs):
        self.request = kwargs.pop('request', None)
        super().__init__(*args, **kwargs)
        self.submit = self.submit or submit
        self.css_class = cssclass

    def as_ht(self):

        csrf_t = 'None'
        submit = self.submit
        cssclass = format_html(u'class="{}"', self.css_class) if len(self.css_class) > 0 else ''

        if self.request != None:
            csrf_t = '<input type="hidden" name="csrfmiddlewaretoken" value="' + get_token(self.request) + '">'
        if self.submit == 'def':
            submit = 'in_' + self.__class__.__name__ + '_not_defined'

        html = '<form method="post" ' + cssclass + '>' + csrf_t + self.as_p() + '<input type="submit" value=' + submit + '></form>'

        return mark_safe(html)

    def __unicode__(self):
        return self.as_h()


class DivHyperModelForm(HyperModelForm):  # (HmlModelForm)
    """
    Делает help_text_html невидимым и добавляет к нему класс help-text
    а так же скрывает реализацию самой формы
    """

    def as_div(self):

        _multi_type = any(isinstance(field, FileField) for field in self.fields.values())

        enctype = "enctype=\"multipart/form-data\"" if _multi_type else ''
        action = self.action
        cssclass = format_html(u'class="{}"', self.css_class) if len(self.css_class) > 0 else ''

        as_p = self._html_output(
            normal_row=u'<div%(html_class_attr)s>%(label)s %(field)s %(help_text)s %(errors)s</div>',
            error_row=u'<div class="error">%s</div>',
            row_ender='</div>',
            help_text_html=u'<div hidden class="help-text">%s</div>',
            errors_on_separate_row=False)
        csrf_t = '<p style="color:red">Set csrf in your view</p>' if self.request == None \
            else '<input type="hidden" name="csrfmiddlewaretoken" value="' + get_token(self.request) + '">'
        submit = '`submit` in ' + self.__class__.__name__ + ' is not defined' if self.submit == 'def' \
            else self.submit

        html = f'<form method="post" {cssclass} {action} {enctype}> ' \
               f'   {csrf_t} {as_p} <input type="submit" value="{submit}">' \
               f'</form>'

        return mark_safe(html)


# example:

"""
class CreatePerson(HyperModelForm):
    # required_css_class = 'required_fields'
    error_css_class = 'error_class'  # работает, но не востребовано

    class Meta(object):
        model = Profile
        fields = (
            'username', 'password', 'first_name', 'last_name', 'email', 'City', 'Sex', 'Age', 'Image')  # exclude
        # exclude = ('username',)
        labels = {
            'username': 'Введите желаемый логин',
            'password': 'Пароль',
            'first_name': 'Имя',
            'last_name': 'Фамилия либо Отчество',
            'email': 'Почта',
            'Image': 'Изображение',
        }
        help_texts = {
            'username': (''),
        }
        widgets = {
            'password': forms.PasswordInput(attrs={'placeholder': u'Пароль'}),
            'email': forms.EmailInput(attrs={'placeholder': 'mail@example.ru'}),
            'Image': forms.FileInput(
                attrs={
                    'accept': "image/jpeg,image/png",
                    'style': 'visibility:hidden',
                    'onchange': 'file_upload(self, event)'}),  # 'style':  display:none
        }
        error_messages = {
            NON_FIELD_ERRORS: {
                'unique_together': "%(model_name)s's %(field_labels)s are not unique.",
                # если нужны будут два уникальных поля
            },
            'username': {
                'unique': u"Пользователь с таким именем уже существует",
            },
        }

    def __init__(self, *args, **kwargs):
        super(CreatePerson, self).__init__(*args, **kwargs)
        self.submit = u'Присоединиться'
        self.css_class = ''

        self.fields['email'].validators = [EmailValidator(message="Некорректный адрес электронной почты")]
        self.fields['Image'] = CustomImageField(self.fields['Image'])
"""



from setuptools import setup, find_packages
import sys, os

version = '1.1.6'
packages = find_packages()
desc = open("README.md").read()

setup(name='django-ui-widgets',
      version=version,
      description="Library of django ui widgets",
      long_description=desc,
      long_description_content_type='text/markdown',
      classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 3.0',
        'Framework :: Django :: 3.1',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Topic :: Software Development :: Build Tools',
      ], # Get strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
      keywords=['django', 'forms', 'widgets', 'fields', 'django-ui'],
      author='Alexander',
      author_email='digital-mag@ya.ru',
      url='https://github.com/Sanshain/django-ui-widgets',
      license='AGPL',
      packages=find_packages(),  # ['django-ui-widgets'],
      include_package_data=True,
      install_requires=["Django>=2.0"],

      entry_points="""
      # -*- Entry points: -*-
      """,
      )
